'use strict';

var _modules = require('./modules');

var _routes = require('./routes');

// app setup
/**
 * Import Node Modules
 */
var app = (0, _modules.express)();

/**
 * Import Routes
 */

var staticpath = _modules.path.join(__dirname, '../node_modules/codepix-client/lib/public');
// view engine setup
app.set('views', _modules.path.join(__dirname, 'views'));
app.set('view engine', 'hbs');

app.use((0, _modules.compression)({ level: 9, filter: shouldCompress }));
app.use(_modules.expressPhantom.SEORender);

function shouldCompress(req, res) {
  if (req.headers['x-no-compression']) {
    // don't compress responses with this request header
    console.log('no compression');
    return false;
  }
  // fallback to standard filter function
  return _modules.compression.filter(req, res);
}

app.use((0, _modules.favicon)(_modules.path.join(staticpath, 'favicon.ico')));
app.use((0, _modules.logger)('dev'));
app.use(_modules.bodyParser.json());
app.use(_modules.bodyParser.urlencoded({ extended: false }));
app.use((0, _modules.cookieParser)());
app.use(_modules.express.static(staticpath));
app.use('/c0dez/data', _modules.express.static('data'));
app.use('/api', function (req, res, next) {
  if (app.get('rasterizer')) {
    req.rasterizer = app.get('rasterizer');
  }
  next();
});
app.use('/api', _routes.api);
app.use('/', _routes.root);
// catch 404 and forward to error handler
app.use(function (req, res, next) {
  var err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
  app.use(function (err, req, res) {
    res.status(err.status || 500);
    res.render('error', {
      message: err.message,
      error: err
    });
  });
}

// production error handler
// no stack-traces leaked to user
app.use(function (err, req, res) {
  res.status(err.status || 500);
  res.render('error', {
    message: err.message,
    error: {}
  });
});

module.exports = app;