'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Rasterizer = undefined;

var _modules = require('./modules');

var _Service2 = require('./Service');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Module dependencies.
 */

var defaults = {
  command: 'phantomjs',
  debug: false,
  host: '127.0.0.1',
  path: '/tmp',
  address: 'http://localhost:3000/api',
  port: 3000
};

/**
 * Private Functions
 */
var toJSON = function toJSON(data) {
  var str = JSON.stringify(data, null, 2);
  JSON.parse(str);
  return str;
};

var toFile = function toFile(filePath) {
  return new Promise(function (resolve, reject) {
    _modules.fs.stat(filePath, function (err, stats) {
      if (err) {
        reject(err);
      } else {
        resolve({
          id: filePath.split('.')[0].split('/')[1],
          path: filePath,
          stats: stats
        });
      }
    });
  });
};

var createMeta = function createMeta(_ref) {
  var code = _ref.code;
  var file = _ref.file;

  return {
    id: file.id,
    card: 'summary_large_image',
    site: '@codepix',
    creator: '@docodemore',
    title: 'Check out teh code',
    description: code,
    url: 'http://codepix.io/c0dez/' + file.path,
    image: 'c0dez/' + file.path,
    createdAt: file.stats.birthtime,
    size: file.stats.size,
    language: _modules.hljs.highlightAuto(code).language
  };
};

var hash = function hash(code) {
  return _modules.crypto.createHash('sha1').update(code).digest('hex');
};

var Rasterizer = exports.Rasterizer = (function (_Service) {
  _inherits(Rasterizer, _Service);

  _createClass(Rasterizer, [{
    key: 'logger',
    value: function logger() {
      if (this.locals.debug) {
        console.log('\n ', _modules.chalk.cyan('Rasterizer:'), _modules.chalk.white.apply(_modules.chalk, arguments), '\n');
      }
    }
  }, {
    key: 'handleStdOut',
    value: function handleStdOut(data) {
      this.logger('service:stdout > ', toJSON(data));
    }
  }, {
    key: 'handleStdErr',
    value: function handleStdErr(data) {
      this.logger('service:stderr > ', toJSON(data));
    }
  }, {
    key: 'handleStdExt',
    value: function handleStdExt(data) {
      this.logger('service:stderr > ', toJSON(data));
    }
  }, {
    key: 'handleStdReq',
    value: function handleStdReq(data) {
      this.logger('service:request > ', toJSON(data));
    }
  }, {
    key: 'spawnPhantom',
    value: function spawnPhantom() {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        _modules.phantom.create(function (service) {
          if (service) {
            _this2.logger('Process created', service.process.pid);
            service.process.stdout.on('data', _this2.handleStdOut);
            service.process.stderr.on('data', _this2.handleStdErr);
            service.process.stderr.on('exit', _this2.handleStdExt);
            resolve(service);
          } else {
            reject('spawnPhantom failed');
          }
        });
      });
    }
  }, {
    key: 'getPayload',
    value: function getPayload(code) {
      return {
        operation: 'POST',
        encoding: 'utf8',
        headers: {
          'Content-Type': 'application/json',
          'x-no-compression': 'true',
          accept: 'application/json',
          connection: 'close',
          'accept-encoding': 'gzip, deflate'
        },
        data: toJSON({ code: code })
      };
    }

    // 6b3c25d7d8918eeda3230357a58ecf5ea20bf5f3

  }, {
    key: 'rasterizeCode',
    value: function rasterizeCode(code) {
      var _this3 = this;

      this.logger('create internal page');
      var payload = this.getPayload(code);
      var lines = code.split(/\r\n|\r|\n/).length;
      var size = { width: 435, height: lines * 20 };
      var filePath = 'data/' + hash(code) + '.png';

      return new Promise(function (resolve, reject) {
        _this3.service.createPage(function (page, createError) {
          if (createError) {
            reject(createError);
          } else {
            try {
              page.set('viewportSize', size);
              _this3.logger('address', _this3.locals.address);
              _this3.logger('payload:\n', toJSON(payload));
              page.open(_this3.locals.address, payload, function (status) {
                _this3.logger('open status:', status);
                if (status === 'success') {
                  page.render(filePath, function () {
                    toFile(filePath).then(function (file) {
                      _this3.logger('file created:\n', toJSON(file));
                      _modules.fs.writeFile(filePath + '.meta.json', toJSON(createMeta({ code: code, file: file })), function (metaError) {
                        if (metaError) {
                          reject(metaError);
                        } else {
                          resolve(createMeta({ code: code, file: file }));
                        }
                      });
                    }).catch(function (err) {
                      return reject(err);
                    });
                  });
                } else {
                  _this3.logger('address', _this3.locals.address);
                  _this3.logger(toJSON(payload));
                  reject(new Error('page open failed'));
                }
              });
            } catch (pageOpenError) {
              reject(pageOpenError);
            }
          }
        });
      });
    }
  }, {
    key: 'startService',
    value: function startService() {
      var _this4 = this;

      this.logger('Starting service...');
      return new Promise(function (resolve, reject) {
        _this4.spawnPhantom().then(function (service) {
          _this4.service = service;
          resolve(_this4);
        }).catch(function (err) {
          return reject(err);
        });
      });
    }
  }, {
    key: 'killService',
    value: function killService() {
      if (this.service) {
        // Remove the exit listener to prevent the rasterizer from restarting
        this.service.process.removeListener('exit', this.handleStdExt);
        this.service.process.kill();
        this.logger('Process killed', this.service.process.pid);
      }
    }
  }]);

  function Rasterizer(options) {
    _classCallCheck(this, Rasterizer);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Rasterizer).call(this));

    _this.locals = Object.assign({}, defaults, options);

    _this.locals.address = 'http://' + options.host + ':' + options.port + '/api';

    process.on('exit', function () {
      _this.logger('Stopping service...');
      _this.killService();
    });

    return _this;
  }

  return Rasterizer;
})(_Service2.Service);